/**
 * @file hv_cal_DynamicColorWin.h
 * @brief Header file of mute module.
 *
 * @verbatim
 * ==============================================================================
 *                        ##### How to use #####
 * ==============================================================================
 *
 * @endverbatim
 *
 * @author HiView SoC Software Team
 * @version 1.0.0
 * @date 2024-09-10
 */

#ifndef __HV_CAL_DYNAMIC_COLOR_WIN_H__
#define __HV_CAL_DYNAMIC_COLOR_WIN_H__
#if (HV_CONFIG_ON == HV_PROJECT_CONFIG_DPU_BRIGHT_FRAME)

#include "hv_comm_DataType.h"

#include "HV_REGS_HT7315.h"
#include "hv_vos_Comm.h"
#include "hv_comm_Define.h"

VOID Hv_Cal_DpuScaler_SetDcwLdgLatch(BOOL bEn);

VOID Hv_Cal_DpuScaler_DcwSrpLatch(BOOL bEn);
VOID Hv_Cal_DpuScaler_DcwReadFreeze(BOOL bEn);

VOID Hv_Cal_DpuScaler_SetDcwSeamlessApplyInEn(BOOL bEn);
VOID Hv_Cal_DpuScaler_DwcLdgTopBlendEn(BOOL bEn);

VOID Hv_Cal_DpuScaler_DcwCapVgsfInOutSize(const VideoConfigParams* pstVideoConfigParams);

VOID Hv_Cal_DpuScaler_DcwCapScalerDownParam(const VideoConfigParams* pstVideoConfigParams);

VOID Hv_Cal_DpuScaler_DcwCapVgsfOutCutParam(DpuCapCutParam* pstDpuCapCutParam);

VOID Hv_Cal_DpuScaler_SetDcwWindowConifg(DpuWindowParam* pstDcwWindowParam);

VOID Hv_Cal_DpuScaler_DcwWvdmWrEnable(BOOL bEn);

VOID Hv_Cal_DpuScaler_DcwRvdmRdEnable(BOOL bEn);

VOID Hv_Cal_DpuScaler_DcwFrcLowLatencyMode(BOOL bEn, UINT32 uiVact);
#endif
#endif
